@@ -19,7 +19,7 @@ test_that("set_threshold()", {
 
 
 test_that("$log() & co work", {
-  ml <- Logger$new("test_logger", appenders = list(memory = AppenderDt$new()))
+  ml <- Logger$new("test_logger", appenders = list(memory = AppenderBuffer$new()))
   ts <- structure(1540486764.41946, class = c("POSIXct", "POSIXt"))
 
   testfun <- function(){
@@ -130,7 +130,7 @@ test_that("add_appenders()/remove_appenders()", {
   # add
   ml$add_appender(app1)
   ml$add_appender(app2, "blah")
-  ml$add_appender(AppenderDt$new(), "blubb")
+  ml$add_appender(AppenderBuffer$new(), "blubb")
   expect_identical(ml$appenders[[2]], app1)
   expect_identical(ml$appenders$blah, app2)
 
@@ -185,18 +185,24 @@ test_that("Exceptions are cought and turned into warnings", {
     )
   )
 
-  expect_warning(ml$fatal(stop("blubb")), "Error.*blubb")
-  expect_warning(ml$fatal(), "Error")
+  expect_warning(ml$fatal(stop("blubb")), "error.*blubb")
+  expect_warning(ml$fatal(), "error")
 })
 
 
 
 
 test_that("Inheritance and event propagation works", {
+  on.exit({
+    c1$config(NULL)
+    c2$config(NULL)
+    c3$config(NULL)
+    unlink(c(tf1, tf2, tf3))
+  })
+
   tf1 <- tempfile()
   tf2 <- tempfile()
   tf3 <- tempfile()
-  on.exit(unlink(c(tf1, tf2, tf3)))
 
   c1  <- get_logger("c1")
   c1$add_appender(AppenderFile$new(tf1))
@@ -221,6 +227,60 @@ test_that("Inheritance and event propagation works", {
 
 
 
+test_that("Named inherited appenders are displayed correctly", {
+  on.exit({
+    c1$config(NULL)
+    c2$config(NULL)
+    c3$config(NULL)
+  })
+
+  c1  <- get_logger("c1")$
+    set_propagate(FALSE)$
+    add_appender(AppenderConsole$new(), "console")
+
+  expect_null(c1$inherited_appenders)
+
+  c2  <- get_logger("c1/c2")$
+    add_appender(AppenderConsole$new(), "console")
+
+  expect_length(c2$inherited_appenders, 1L)
+
+  c3  <- Logger$new("c1/c2/c3")$
+    add_appender(AppenderConsole$new(), "console")
+
+  expect_length(c3$inherited_appenders, 2L)
+
+  expect_identical(names(c3$inherited_appenders), c("console", "console"))
+  expect_output(print(c3), "\\sconsole:.*\\sconsole:.*")
+})
+
+
+
+
+test_that("Unnamed inherited appenders are displayed correctly", {
+  on.exit({
+    c1$config(NULL)
+    c2$config(NULL)
+    c3$config(NULL)
+  })
+
+  c1  <- get_logger("c1")$
+    set_propagate(FALSE)$
+    add_appender(AppenderConsole$new())
+
+  c2  <- get_logger("c1/c2")$
+    add_appender(AppenderConsole$new())
+
+  c3  <- Logger$new("c1/c2/c3")$
+    add_appender(AppenderConsole$new())
+
+  expect_null(names(c3$inherited_appenders))
+  expect_output(print(c3), "\\[\\[1\\]\\]\\:.*\\[\\[2\\]\\]\\:.*")
+})
+
+
+
+
 test_that("threshold works", {
   c1  <- Logger$new("c1")
   expect_output(c1$error("blubb"), "ERROR")
@@ -299,3 +359,37 @@ test_that("$config works with lists", {
   expect_true(is_virgin_Logger(l, allow_subclass = TRUE))
   expect_false(is_virgin_Logger(l))
 })
+
+
+
+
+
+test_that("Logger$log() dispatches to all appenders, even if some throw an error", {
+  ln <- Logger$new("normal", propagate = FALSE)
+  lg <- LoggerGlue$new("glue", propagate = FALSE)
+
+  AppErr <- R6::R6Class(
+    inherit = AppenderConsole,
+    public = list(
+      append = function(...) stop("error")
+    )
+  )
+
+  tf <- tempfile()
+  on.exit(unlink(tf))
+
+  ln$set_appenders(list(
+    err = AppErr$new(),
+    file = AppenderFile$new(file = tf)
+  ))
+  lg$set_appenders(list(
+    err = AppErr$new(),
+    file = AppenderFile$new(file = tf)
+  ))
+
+  expect_warning(ln$info("test_normal"), class = "appendingFailedWarning")
+  expect_warning(ln$info("test_glue"), class = "appendingFailedWarning")
+
+  expect_true(any(grepl("test_normal", readLines(tf))))
+  expect_true(any(grepl("test_glue", readLines(tf))))
+})
